/*
 * This file belongs to FreeMiNT. It's not in the original MiNT 1.12
 * distribution. See the file CHANGES for a detailed log of changes.
 * 
 * 
 * Copyright 2000, 2001, 2002 Frank Naumann <fnaumann@freemint.de>
 * All rights reserved.
 * 
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * 
 * begin:	2000-01-01
 * last change:	2000-03-07
 * 
 * Author:	Frank Naumann <fnaumann@freemint.de>
 * 
 * Please send suggestions, patches or bug reports to me or
 * the MiNT mailing list.
 * 
 * 
 * changes since last version:
 * 
 * known bugs:
 * 
 * todo:
 * 
 * optimizations:
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "main.h"
#include "list.h"
#include "generate.h"
#include "traps.h"


#ifdef __MINT__
long _stksize = 64 * 1024;
#endif

int errors = 0;

/* from parser */
int yyparse(void);

/* from parser */
extern FILE *yyin, *yyout;


void
print_head(FILE *out, const char *myname)
{
	time_t t = time(NULL);
	
	fprintf(out, "/*\n * %s, %s *\n", myname, ctime (&t));
	fprintf(out,
		" * DO NOT EDIT - this file is automatically generated\n"
		" */\n"
	);
	fprintf(out, "\n");
	
	fprintf(out,
		"/*                                                                      \n"
		" * This file belongs to FreeMiNT. It's not in the original MiNT 1.12    \n"
		" * distribution. See the file CHANGES for a detailed log of changes.    \n"
		" *                                                                      \n"
		" *                                                                      \n"
		" * Copyright 2000, 2001, 2002 Frank Naumann <fnaumann@freemint.de>      \n"
		" * All rights reserved.                                                 \n"
		" *                                                                      \n"
		" * This file is free software; you can redistribute it and/or modify    \n"
		" * it under the terms of the GNU General Public License as published by \n"
		" * the Free Software Foundation; either version 2, or (at your option)  \n"
		" * any later version.                                                   \n"
		" *                                                                      \n"
		" * This file is distributed in the hope that it will be useful,         \n"
		" * but WITHOUT ANY WARRANTY; without even the implied warranty of       \n"
		" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        \n"
		" * GNU General Public License for more details.                         \n"
		" *                                                                      \n"
		" * You should have received a copy of the GNU General Public License    \n"
		" * along with this program; if not, write to the Free Software          \n"
		" * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            \n"
		" *                                                                      \n"
		" */                                                                     \n"
	);
	fprintf(out, "\n");
}

static void
usage(void)
{
	fprintf(stderr, "Usage: \n");
	exit(2);
}

int
main(int argc, char **argv)
{
	const char *myname;
	
	myname = *argv++;
	
#if 0
	while (*argv)
	{
		if (!strcmp(*argv, "-o"))
		{
			argv++;
			if (*argv == 0)
			{
				fprintf(stderr, "missing argument to -o\n");
				usage();
			}
			
			f = fopen(*argv, "wt");
			if (!f)
				perror(*argv);
			else
				outfile = f;
			
			argv++;
		}
		else
		{
			if (**argv == '-')
			{
				fprintf(stderr, "unknown option: %s\n", *argv);
				usage();
			}
		}
	}
#endif
	
	
	if (*argv == 0)
	{
#if YYDEBUG != 0
		yydebug = 1;
#endif
		yyparse();
	}
	else
	{
		while (*argv)
		{
			FILE *f;
			
			f = fopen(*argv, "rt");
			if (f)
			{
				yyin = f;
				yyparse();
				
				fclose(f);
			}
			else
			{
				perror(*argv);
				exit(1);
			}
			
			argv++;
		}
	}
	
	if (!errors)
	{
		FILE *f;
		
		/* check_tab(); */
		
		f = fopen("../include/mint/sysbind.h", "w");
		if (!f)
		{
			perror("sysbind.h");
			exit(1);
		}
		
		print_head(f, myname);
		fprintf(f, "#ifndef _sysbind_h\n");
		fprintf(f, "#define _sysbind_h\n\n");
		fprintf(f, "#include <sys/types.h>\n");
		fprintf(f, "#include <mint/trap.h>\n");
		fprintf(f, "\n");
		fprintf(f, "\n");
		
		generate_bindings_old(f, DOS);
		generate_bindings_impl(f, DOS);
		
		fprintf(f, "\n#endif /* _sysbind_h */\n");
		fclose(f);
		
		generate_trap_h("../include/mint");
		generate_traps(".");
	}
	
	return errors;
}
